package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.Objects;
import com.google.common.base.Preconditions;
import net.minecraft.tags.DamageTypeTags;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.ChatFormatting;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.placebo.util.StepFunction;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.DamageReductionAffix;

public class DamageReductionBonus extends GemBonus
{
    protected final DamageReductionAffix.DamageType type;
    protected final Map<LootRarity, StepFunction> values;
    public static Codec<DamageReductionBonus> CODEC;
    
    public DamageReductionBonus(final GemClass gemClass, final DamageReductionAffix.DamageType type, final Map<LootRarity, StepFunction> values) {
        super(Apotheosis.loc("damage_reduction"), gemClass);
        this.type = type;
        this.values = values;
    }
    
    @Override
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        final float level = this.values.get(rarity).get(0.0f);
        return (Component)Component.m_237110_("affix.apotheosis:damage_reduction.desc", new Object[] { Component.m_237115_("misc.apotheosis." + this.type.getId()), Affix.fmt(100.0f * level) }).m_130940_(ChatFormatting.YELLOW);
    }
    
    @Override
    public float onHurt(final ItemStack gem, final LootRarity rarity, final DamageSource src, final LivingEntity user, final float amount) {
        if (!src.m_269533_(DamageTypeTags.f_268738_) && !src.m_269533_(DamageTypeTags.f_268413_) && this.type.test(src)) {
            final float level = this.values.get(rarity).get(0.0f);
            return amount * (1.0f - level);
        }
        return super.onHurt(gem, rarity, src, user, amount);
    }
    
    @Override
    public GemBonus validate() {
        Preconditions.checkNotNull((Object)this.type, (Object)"Invalid DamageReductionBonus with null type");
        Preconditions.checkNotNull((Object)this.values, (Object)"Invalid DamageReductionBonus with null values");
        Preconditions.checkArgument(this.values.entrySet().stream().mapMulti((entry, consumer) -> {
            consumer.accept(entry.getKey());
            consumer.accept(entry.getValue());
            return;
        }).allMatch(Objects::nonNull), (Object)"Invalid DamageReductionBonus with invalid values");
        return this;
    }
    
    @Override
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    @Override
    public int getNumberOfUUIDs() {
        return 0;
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return DamageReductionBonus.CODEC;
    }
    
    static {
        DamageReductionBonus.CODEC = (Codec<DamageReductionBonus>)RecordCodecBuilder.create(inst -> inst.group((App)GemBonus.gemClass(), (App)DamageReductionAffix.DamageType.CODEC.fieldOf("damage_type").forGetter(a -> a.type), (App)DamageReductionBonus.VALUES_CODEC.fieldOf("values").forGetter(a -> a.values)).apply((Applicative)inst, DamageReductionBonus::new));
    }
}
